<?php

namespace phil\HabboFont2\artist;

use phil\HabboFont2\addons\PositionType;
use phil\HabboFont2\addons\zIndex;
use phil\HabboFont2\cache\Cache;
use phil\HabboFont2\Config;
use phil\HabboFont2\fonts\Font;

class Artist
{

    private $_text;

    private $_font;

    private $_save;

    private $_path;

    private $_backgroundAddons = [];

    private $_foregroundAddons = [];

    public function setText($text)
    {

        $this->_text = strtoupper($text);

        return $this;
    }

    /**
     * @return mixed
     */
    public function getFont()
    {

        return $this->_font;
    }

    public function setFont(Font $font)
    {

        $this->_font = $font;

        return $this;
    }

    public function save($path, $bool = Config::SAVE_IMAGES)
    {

        $this->_path = $path;
        $this->_save = $bool;

        return $this;
    }

    public function execute($callback)
    {

        if (!$this->_font->isTextValid($this->_text)) {
            exit("Invalid Word");
        }
        $this->_text = $this->_font->getTextWithReplacements($this->_text);

        $cache = Cache::get();

        if (($cache_image = $cache->getByInfo([
                "text" => $this->_text,
                "addons" => $this->_font->getActiveAddons(),
                "font" => $this->_font
            ])) == null
        ) {

            $drawer = new Drawer();
            $chars = new Chars($this);
            $chars->setChars($this->_text);

            $width = $chars->getWidth(-$this->_font->getBorder()) + $this->_font->getBorder();
            $height = $chars->getHeight();

            $original_width = $width;
            $original_height = $height;

            $char_padding_left = 0;
            $char_padding_top = 0;

            foreach ($this->getFont()->getActiveAddons() as $addon) {
                $position_type = $addon->getPositionType();
                $x = $addon->getX();
                $y = $addon->getY();

                if ($addon->zIndex() == zIndex::BACKGROUND) {
                    $this->_backgroundAddons[] = $addon;
                } else {
                    if ($addon->zIndex() == zIndex::FOREGROUND) {
                        $this->_foregroundAddons[] = $addon;
                    }
                }

                switch ($position_type) {
                    case PositionType::TOP_LEFT:
                    case PositionType::TOP_FIRST_CHAR:
                        if ($x < 0 && $original_width - $x > $width) {
                            $width = $original_width - $x;
                            $char_padding_left = $width - $original_width;
                        }
                        if ($y < 0 && $original_height - $y > $height) {
                            $height = $original_height - $y;
                            $char_padding_top = $height - $original_height;
                        }
                        break;

                    case PositionType::TOP_LAST_CHAR:
                        if ($y < 0 && $original_height - $y > $height) {
                            $height = $original_height - $y;
                            $char_padding_top = -$y;
                        }
                        break;
                }
            }

            foreach ($this->getFont()->getActiveAddons() as $addon) {
                $position_type = $addon->getPositionType();
                $x = $addon->getX();
                $y = $addon->getY();

                if ($position_type == PositionType::TOP_LEFT || $position_type == PositionType::TOP_LAST_CHAR) {
                    if ($position_type == PositionType::TOP_LEFT) {
                        if ($x < 0 && $original_width - $x == $width) {
                            $addon->setX(0);
                        } else {
                            $_x = $x < 0 ? -$x : $x;
                            $addon->setX($char_padding_left - $_x);
                        }
                    }

                    if ($y < 0 && $original_height - $y == $height) {
                        $addon->setY(0);
                    } else {
                        $_y = $y < 0 ? -$y : $y;
                        $addon->setY($char_padding_top - $_y);
                    }
                }

                if ($position_type == PositionType::TOP_LAST_CHAR) {
                    $position_x = $char_padding_left + $addon->getX();
                    for ($i = 0; $i < count($chars->getChars()); ++$i) {
                        $char = $chars->getChars()[$i];
                        $position_x += $char->getWidth() - $this->_font->getBorder();
                    }

                    $addon->setX($position_x - $this->_font->getBorder() - ($addon->getWidth() / 2));
                }

                if ($position_type == PositionType::TOP_FIRST_CHAR) {
                    $addon->setX($char_padding_left + $addon->getX());
                    $addon->setY($char_padding_top + $addon->getY());
                }

            }

            $image = $drawer->createBlankImage($width, $height);

            // background addons
            foreach ($this->_backgroundAddons as $addon) {
                $image->appendImage(new AddonImage($addon), $addon->getX(), $addon->getY());
            }

            foreach ($chars->getChars() as $char) {
                $image->appendImage(new CharImage($char), $char_padding_left, $char_padding_top);
                $char_padding_left += $char->getWidth() - $this->_font->getBorder();
            }

            // foreground addons
            foreach ($this->_foregroundAddons as $addon) {
                $image->appendImage(new AddonImage($addon), $addon->getX(), $addon->getY());
            }


            if ($this->_save) {
                do {
                    $cache_image = $cache->save($image, $this->_font, $this->_text);
                } while ($cache_image == null);
            }

            return $callback(new ArtistResult($image));

        } else {
            return $callback(new ArtistResult($cache_image));
        }
    }
}