<?php

namespace phil\HabboFont2\fonts;

use phil\HabboFont2\addons\Addon;

abstract class Font {

	abstract public function getDirectory();

	abstract public function getBorder();

	abstract public function getCharSigns();

	abstract public function getCharReplacements();

	private $_addons = [];

	private $_activeAddons = [];

	public function addAddon($addon) {

		$addon = new $addon;
		$this->_addons[] = $addon;
	}

	private function getAddon($class_name) {

		foreach ($this->_addons as $addon) {
			if ($addon instanceof $class_name) {
				return $addon;
			}
		}

		return null;
	}

	public function withAddons(array $addons) {

		foreach ($addons as $addon) {
			$_addon = $this->getAddon($addon);
			if ($_addon != null && in_array($_addon, $this->_addons) && !in_array($_addon, $this->_activeAddons)) {
				$this->_activeAddons[] = $_addon;
			}
		}

		return $this;
	}

	public static function trim($text) {

		do {
			$replaced_something = false;

			$new_text = str_replace("  ", " ", $text);
			$replaced_something = $new_text != $text;

			$text = $new_text;
		} while ($replaced_something);

		return $text;
	}

	public function isTextValid($text) {

		$text = strtoupper($text);

		for ($i = 0; $i < strlen($text); $i++) {
			if (!in_array($text[ $i ], $this->getCharSigns())) {
				return false;
			}
		}

		return true;
	}

	public function getTextWithReplacements($text) {

		for ($i = 0; $i < strlen($text); $i++) {
			if (isset($this->getCharReplacements()[ $text[ $i ] ])) {
				$text[ $i ] = $this->getCharReplacements()[ $text[ $i ] ];
			}
		}

		return $text;
	}

	/**
	 * @return array
	 */
	public function getAddons() {

		return $this->_addons;
	}

	public function getActiveAddons() {

		return $this->_activeAddons;
	}
}